namespace eval mail_job {

    ad_proc -public send {
	-mail_job_id:required
    } {
	Send email
    } {
	set boundary_string "-+-+-someboundarystring-+-+-"
	set alternative_string "-+-+-somealternativestring-+-+-"
	set system_url [ad_url]
	regsub -all {http://} $system_url {} system_url

	# get the mail job data
	if {![db_0or1row get_data {}]} {
	    return
	}

	# mark mail job as started
	db_dml start_mail_job {}

	# glue body and footer together
	if {$mail_mime_type != "text/html"} {
	    set text_body "$text_header\n$text_body\n$text_footer"
	}
	if {$mail_mime_type != "text/plain"} {
	    set html_body "$html_header\n$html_body\n$html_footer"
	}

	# get endofline characters right
	set text_body [string map {\r ""} $text_body]
	set html_body [string map {\r ""} $html_body]

	# substitute links in mail if link tracking enabled
	if {$track_links_p == "t"} {
	    # Tracking links in plain text email looks really tacky
# 	    if {$mail_mime_type != "text/html"} {
# 		set text_body [mail_job::create_links -text $text_body -mime_type text/plain -mail_job_id $mail_job_id]
# 	    }
	    if {$mail_mime_type != "text/plain"} {
		set html_body [mail_job::create_links -text $html_body -mime_type text/html -mail_job_id $mail_job_id]
	    }
	    set template_p t
	}

	if {$mail_mime_type != "text/plain"} {
	    # substitute image link to track viewing the mail
	    regsub -all -nocase {(@view_mail_link@)} $html_body "<img src=\"[ad_url][mailing_list::util::get_sw_url -package_key mail-links]view/$mail_job_id\.@user_id@\" width=\"1\" height=\"1\">" html_body
	}

	set attachments ""
	# encode all attachments in base64
	db_foreach get_attachments {} {
	    set mail_filename "\"[ad_quotehtml $title]\""

	    # encode current attachment in base64 and wrap it in
	    # appropriate email-multipart form
	    set content [mail_job::base64_encode -filename "[cr_fs_path]$filename"]

	    append attachments "\n--$boundary_string
Content-Type: $mime_type; name=$mail_filename
Content-Transfer-Encoding: base64
Content-Disposition: attachment; filename=$mail_filename

$content\n"
	}

	set header [ns_set new]
	if {![empty_string_p $attachments]} {
	    # set correct header and correct begin/end of multipart-mail

	    # finish multipart email
	    append attachments "\n--$boundary_string\--"

	    # begin multipart/mixed email.
	    ns_set put $header "Content-Type" "Multipart/Mixed; boundary=\"$boundary_string\""

	    set begin_body "This is a multi-part message in MIME format.\n\n--$boundary_string\n"
	    switch $mail_mime_type {
		"text/plain" {
		    # quoted-printable encode plaintext mails
		    append begin_body "Content-Type: text/plain\nContent-Disposition: inline\n\n"
######
#		    append begin_body "Content-Type: text/plain\nContent-Transfer-Encoding: quoted-printable\nContent-Disposition: inline\n\n"
######
		}
		"text/html" {
		    append begin_body "Content-Type: text/html; charset=$mime_charset\nContent-Disposition: inline\n\n"
		}
		"multipart/alternative" {
		    # begin multipart/alternative part inside the multipart/mixed mail
		    append begin_body "Content-Type: multipart/alternative; boundary=\"$alternative_string\"\n\n--$alternative_string\nContent-Type: text/plain"
######
#		    append begin_body "Content-Type: multipart/alternative; boundary=\"$alternative_string\"\n\n--$alternative_string\nContent-Type: text/plain\nContent-Transfer-Encoding: quoted-printable"
######
		    set alternative_body "\n--$alternative_string\nContent-Type: text/html; charset=$mime_charset\n"
		    set attachments "\n--$alternative_string\--\n$attachments"
		}
	    }

	} else {
	    # no attachments, so set correct header
	    switch $mail_mime_type {
		"text/plain" {
		    # quoted-printable encode plaintext mails
		    ns_set put $header "Content-Type" text/plain
######
#		    ns_set put $header "Content-Transfer-Encoding" quoted-printable
######
		    set begin_body ""
		}
		"text/html" {
		    ns_set put $header "Content-Type" "text/html; charset=$mime_charset"
		    set begin_body ""
		}
		"multipart/alternative" {
		    # begin multipart/alternative email
		    ns_set put $header "Content-Type" "Multipart/Alternative; boundary=\"$boundary_string\""
		    # quoted-printable encode plaintext mails
		    set begin_body "This is a multi-part message in MIME format.\n\n--$boundary_string\nContent-Type: text/plain"
######
#		    set begin_body "This is a multi-part message in MIME format.\n\n--$boundary_string\nContent-Type: text/plain\nContent-Transfer-Encoding: quoted-printable"
######
		    set alternative_body "\n--$boundary_string\nContent-Type: text/html; charset=$mime_charset\n"
		    set attachments "\n--$boundary_string\--"
		}
	    }
	}

	set mail_text_body ""
	set mail_html_body ""

	if {$template_p == "t"} {
	    # mail is template so generate code to substitute variables in mail text

	    # we have to use noquote-substitution in mail subject
	    while {[regsub -all [template::adp_variable_regexp] $subject {\1@one_user.\2;noquote@} subject]} {}

	    if {$mail_mime_type != "text/html"} {
		# we have to use noquote-substitution for plaintext emails
		while {[regsub -all [template::adp_variable_regexp] $text_body {\1@one_user.\2;noquote@} text_body]} {}
	    }

	    if {$mail_mime_type != "text/plain"} {
		# we have to use noquote-substitution for mail view tracking in html mails
		while {[regsub -all -nocase {@view_mail_link@} $html_body {@view_mail_link;noquote@} html_body]} {}
		# we have to correct for correct variable usage in html emails
		# (already done for subject and plaintext emails)
		while {[regsub -all [template::adp_variable_regexp] $html_body {\1@one_user.\2@} html_body]} {}
		while {[regsub -all [template::adp_variable_regexp_noquote] $html_body {\1@one_user.\2;noquote@} html_body]} {}
	    }

	    ad_conn -set package_key mailing-lists
	    variable ::template::parse_level
	    lappend ::template::parse_level [info level]
	    switch $mail_mime_type {
		"text/plain" {
		    # generate proc to substitute variables in plaintext mails
		    proc ::mailing_list::__substitute_mail {} "
		    uplevel {
			[template::adp_compile -string $subject]
			set mail_subject \$__adp_output
			[template::adp_compile -string $text_body]
			set mail_text_body \$__adp_output
		    }
		"
		}
		"text/html" {
		    # generate proc to substitute variables in html mails
		    proc ::mailing_list::__substitute_mail {} "
		    uplevel {
			[template::adp_compile -string $subject]
			set mail_subject \$__adp_output
			[template::adp_compile -string $html_body]
			set mail_html_body \$__adp_output
		    }
		"
		}
		"multipart/alternative" {
		    # generate proc to substitute variables in alternative mails
		    proc ::mailing_list::__substitute_mail {} "
		    uplevel {
			[template::adp_compile -string $subject]
			set mail_subject \$__adp_output
			[template::adp_compile -string $text_body]
			set mail_text_body \$__adp_output
			[template::adp_compile -string $html_body]
			set mail_html_body \$__adp_output
		    }
		"
		}
	    }
	    template::util::lpop ::template::parse_level
	} else {
	    # mail is no template, so encode once and not for every user
	    set mail_subject $subject

	    # encode plain-text mails
	    if {$mail_mime_type != "text/html"} {
		set mail_text_body $text_body
######
#		set mail_text_body [mail_job::quoted_printable_encode -text $text_body]
######
	    }
	    set mail_html_body $html_body
	}

	# get all users to spam from mailing-list / users-selection
	if {[empty_string_p $list_id] && ![empty_string_p $bind_vars]} {
	    # get users from users-selection with bind-vars
	    set user_list [db_list_of_ns_sets get_users_selection_to_mail_bind $sql_query -bind $bind_vars]
	} else {
	    if {![empty_string_p $list_id]} {
		# get users from mailing-list
		set user_list [db_list_of_ns_sets get_list_users_to_mail {}]
	    } else {
		# get users from users-selection without bind-vars
		set user_list [db_list_of_ns_sets get_users_selection_to_mail $sql_query]
	    }
	}

	set unsubscribe_link "[ad_url][lindex [site_node::get_url_from_object_id -object_id $package_id] 0]"
	set mail_count 0
	# loop over all users to spam
	foreach one_user_set $user_list {
	    incr mail_count
	    array set one_user [template::util::set_to_list $one_user_set]
	    set user_id $one_user(user_id)
	    set user_email $one_user(user_email)
	    set user_first_names $one_user(user_first_names)
	    set user_last_name $one_user(user_last_name)
	    set user_password $one_user(user_password)
	    set user_salt $one_user(user_salt)

	    if {[string equal $user_first_names "first_names"]} {
		set user_first_names ""
		set one_user(user_first_names) ""
	    }
	    if {[string equal $user_last_name "last_name"]} {
		set user_last_name ""
		set one_user(user_last_name) ""
	    }

	    if {$template_p == "t"} {
		# generate link-variable
		# so that users are automatically logged in to unsubscribe
		set x [ns_sha1 "$user_id $user_password $user_salt"]
		set one_user(x) $x
		set one_user(link) "$unsubscribe_link?[export_url_vars user_id x]"

		# now substitute variables in mailbody
		::mailing_list::__substitute_mail

		# encode plain-text mails
		if {$mail_mime_type != "text/html"} {
######
#		    set mail_text_body [mail_job::quoted_printable_encode -text $mail_text_body]
######
		}
	    }
	    ns_set update $header "X-Field" "$user_id-$mail_job_id"

	    # set the data for mail recipient
	    set to_addr(email) [list $user_email]
	    set to_addr(user_id) [list $user_id]

	    if {[empty_string_p $user_last_name]} {
		set to_addr(name) [list $user_first_names]
	    } else {
		set to_addr(name) [list "$user_first_names $user_last_name"]
	    }

	    #actually send out the email
	    switch $mail_mime_type {
		"text/plain" {
		    acs_mail_lite::send -valid_email -to_addr [array get to_addr] -from_addr $sender_email -subject $mail_subject -body "$begin_body\n$mail_text_body\n$attachments" -extraheaders $header -package_id $package_id
######
#		    acs_mail_lite::send -valid_email -to_addr [array get to_addr] -from_addr $sender_email -subject [mail_job::quoted_printable_encode -text $mail_subject -charset $mime_charset] -body "$begin_body\n$mail_text_body\n$attachments" -extraheaders $header -package_id $package_id
######
		}
		"text/html" {
		    acs_mail_lite::send -valid_email -to_addr [array get to_addr] -from_addr $sender_email -subject $mail_subject -body "$begin_body\n$mail_html_body\n$attachments" -extraheaders $header -package_id $package_id
######
#		    acs_mail_lite::send -valid_email -to_addr [array get to_addr] -from_addr $sender_email -subject [mail_job::quoted_printable_encode -text $mail_subject -charset $mime_charset] -body "$begin_body\n$mail_html_body\n$attachments" -extraheaders $header -package_id $package_id
######
		}
		"multipart/alternative" {
		    acs_mail_lite::send -valid_email -to_addr [array get to_addr] -from_addr $sender_email -subject $mail_subject -body "$begin_body\n$mail_text_body\n$alternative_body\n$mail_html_body\n$attachments" -extraheaders $header -package_id $package_id

######
#		    acs_mail_lite::send -valid_email -to_addr [array get to_addr] -from_addr $sender_email -subject [mail_job::quoted_printable_encode -text $mail_subject -charset $mime_charset] -body "$begin_body\n$mail_text_body\n$alternative_body\n$mail_html_body\n$attachments" -extraheaders $header -package_id $package_id
######
		}
	    }
	    array unset to_addr

	    # record mail sending in logtables
	    db_dml log_mail_sending {}
	    db_dml log_users_latest_mail {}
	}

	# mark mail job as done and save number of mails sent out
	# and set link tracking flag if mail containts tracked links
	if {$mail_count>0} {
	    db_dml finish_mail_job {}

	    array unset one_user
	}
    }

    ad_proc -private base64_encode {
	-filename:required
    } {
        set mmencode_path [acs_mail_lite::get_parameter -name MMEncodeBin -default "/usr/bin/mmencode"]
	set fp [open "|$mmencode_path -b $filename" r]
	set quoted [read $fp]
	close $fp
	return $quoted
    }


    ad_proc -private quoted_printable_encode { 
	-text:required
	{-charset ""}
    } {
	Encode special characters, like german umlauts, in message headers 
	according to RFC 2047.
    } {
	# First check if there are any characters which need to be quoted at all.
	set encode_p 0
	for { set i 0 } { $i < [string length $text] } { incr i } {
	    if { ![string is ascii [string index $text $i]] } {
		set encode_p 1
	    }
	}
	if { !$encode_p } { return $text }

	set hex "0123456789ABCDEF"

	if {[empty_string_p $charset]} {
	    set end_of_line "="
	    set begin_of_line ""
	} else {
	    set end_of_line "?="
	    set begin_of_line "=?$charset?Q?"
	}

	set result ""
	set line $begin_of_line
	for { set i 0 } { $i < [string length $text] } { incr i } {
	    set current [string index $text $i]
	    if { ![string is ascii $current] || [string first $current " \t\r\n()<>@,;:/\[\]?.=\"\\"] != -1 } {
		binary scan $current c x
		append line "=[string index $hex [expr ($x & 0xf0) >> 4]][string index $hex [expr $x & 0x0f]]"
	    } else {
		append line $current
	    }
	    if { [string length $line] > 70 } {
		if { ![empty_string_p $result] } { append result "\n" }
		append result "${line}$end_of_line"
		set line $begin_of_line
	    }
	}

	if { ![string equal $line $begin_of_line] } {
	    if { ![empty_string_p $result] } { append result "\n" }
	    append result "${line}$end_of_line"
	}

	return $result
    }

    ad_proc -public send_mail {
	-to_addr:required
	-from_addr:required
	-subject:required
	-body:required
	-mime_type:required
	-charset:required
	{-package_id ""}
    } {
	send one email
    } {
	if {[empty_string_p $package_id]} {
	    set package_id [ad_conn package_id]
	}
	set header [ns_set new]
	switch $mime_type {
	    "text/plain" {
		# quoted-printable encode plaintext mails
		ns_set put $header "Content-Type" text/plain
######
#		ns_set put $header "Content-Transfer-Encoding" quoted-printable
#		set body [mail_job::quoted_printable_encode -text $body]
######
	    }
	    "text/html" {
		ns_set put $header "Content-Type" "text/html; charset=$charset"
	    }
	}

	acs_mail_lite::send -to_addr $to_addr -from_addr $from_addr -subject $subject -body $body -extraheaders $header -package_id $package_id
######
#	acs_mail_lite::send -to_addr $to_addr -from_addr $from_addr -subject [mail_job::quoted_printable_encode -text $subject -charset $charset] -body $body -extraheaders $header -package_id $package_id
######
    }


    ad_proc -private mail_bounced { header body } {
	Record bounced mail
    } {
	ns_log notice "mailing lists: try to record bounced mail"
	catch {
	    if {[regexp {X-Field: ([0-9]*)-([0-9]*)} $body match user_id job_id]} {
		db_transaction {
		    ns_log notice "mailing lists: bounce recorded for job $job_id and user $user_id"
		    db_dml record_bounce_in_log {}

		    if {[db_resultrows]} {
			db_dml increment_bounce_count {}
		    }
		}
	    }
	}
    }

    ad_proc -private create_links {
	-text:required
	-mime_type:required
	-mail_job_id:required
    } {
	Grab all the links in the given text, insert them into the db and replace
	the links in the text with links to the link-tracking package
    } {
	if {$mime_type == "text/plain"} {
	    set expression {(http://[a-zA-Z0-9:;_/&=,\#\?\.\-]+)}
	    set server_url_substitution "[ad_url][mailing_list::util::get_sw_url -package_key mail-links]\\1"
	} else {
	    set expression {(href|HREF)="(http://[^"]*)"}
            set server_url_substitution "href=\"[ad_url][mailing_list::util::get_sw_url -package_key mail-links]\\1\""
        }

        while {[regexp $expression $text match href url]} {
	    set link_id [mail_link::add -mail_id $mail_job_id -url $url]
	    regsub $expression $text "<TeMpPlAcEhOlDeR>$link_id\.$mail_job_id\.@user_id@</TeMpPlAcEhOlDeR>" text
        }

        regsub -all {<TeMpPlAcEhOlDeR>([^<]*)</TeMpPlAcEhOlDeR>} $text $server_url_substitution text

        return $text
    }

    ad_proc -private sweeper {} {
	Periodically send out all waiting mail jobs
    } {
	# Make sure that only one thread is processing the queue at a time.
	if {[nsv_incr mailing_lists send_mails_p] > 1} {
	    nsv_incr mailing_lists send_mails_p -1
	    return
	}

	acs_mail_lite::with_finally -code {
	    set jobs [db_list get_queued_mail_jobs {}]

	    foreach job_id $jobs {
		send -mail_job_id $job_id
	    }
	} -finally {
	    nsv_incr mailing_lists send_mails_p -1
	}
    }
}
