ad_page_contract {
} {
    class_id:integer,optional
    copy_class_id:integer,optional
    mime_type:optional
}

set user_id [auth::require_login]
set package_id [ad_conn package_id]
permission::require_permission -object_id $package_id -privilege mailing_list_admin

if {[info exists class_id]} {
    set action Edit
} else {
    set action Add
}

if {[info exists class_id] || [info exists copy_class_id]} {
    if {[info exists copy_class_id]} {
	set class_or_copy_id $copy_class_id
    } else {
	set class_or_copy_id $class_id
    }
    if {[db_0or1row get_old_class_mime_type {}]} {
	set css_file [db_list_of_lists get_css_file {}]
    }
}

set context_bar [list [list "." Administration] "$action Mail Class"]

set languages [db_list_of_lists get_ad_locales {}]

ad_form -name mail_class_form -html {enctype multipart/form-data} -action classes -export {copy_class_id} -form {
    {class_id:key}
    {mime_type:text(hidden)}
    {name:text {label "Name"} {html {size 50 maxlength 1000}}}
    {locale:text(select) {label "Language"} {options $languages}}
    {sender_email:text {label "Sender Email"} {html {size 80 maxlength 1000}}}
    {subject:text {label "Default Subject"} {html {size 80 maxlength 1000}}}
    {subject_change_p:text(radio) {label "Subject changeable?"} {options {{"Yes" t} {"No" f}}}}
}

switch $mime_type {
    "multipart/alternative" {
	# alternative mail-class: plaintext and html
	ad_form -extend -name mail_class_form -form {
	    {text_header:text(textarea),optional {label "Default Text Header"} {html {rows 5 cols 80}}}
	    {text_header_change_p:text(radio) {label "Text Header changeable?"} {options {{"Yes" t} {"No" f}}}}
	    {text_body:text(textarea),optional {label "Default Text Body"} {html {rows 5 cols 80}}}
	    {text_body_change_p:text(radio) {label "Text Body changeable?"} {options {{"Yes" t} {"No" f}}}}
	    {text_footer:text(textarea),optional {label "Default Text Footer"} {html {rows 5 cols 80}}}
	    {text_footer_change_p:text(radio) {label "Text Footer changeable?"} {options {{"Yes" t} {"No" f}}}}
	    {html_header:text(textarea),optional {label "Default HTML Header"} {html {rows 5 cols 80}}}
	    {html_header_change_p:text(radio) {label "HTML Header changeable?"} {options {{"Yes" t} {"No" f}}}}
	    {html_body:text(textarea),optional {label "Default HTML Body"} {html {rows 5 cols 80}}}
	    {html_body_change_p:text(radio) {label "HTML Body changeable?"} {options {{"Yes" t} {"No" f}}}}
	    {html_footer:text(textarea),optional {label "Default HTML Footer"} {html {rows 5 cols 80}}}
	    {html_footer_change_p:text(radio) {label "HTML Footer changeable?"} {options {{"Yes" t} {"No" f}}}}
	    {upload_file:file,optional {label "CSS-File"}}
	}
    }
    "text/plain" {
    # plaintext content mail-class
	ad_form -extend -name mail_class_form -form {
	    {text_header:text(textarea),optional {label "Default Text Header"} {html {rows 5 cols 80}}}
	    {text_header_change_p:text(radio) {label "Text Header changeable?"} {options {{"Yes" t} {"No" f}}}}
	    {text_body:text(textarea),optional {label "Default Text Body"} {html {rows 5 cols 80}}}
	    {text_body_change_p:text(radio) {label "Text Body changeable?"} {options {{"Yes" t} {"No" f}}}}
	    {text_footer:text(textarea),optional {label "Default Text Footer"} {html {rows 5 cols 80}}}
	    {text_footer_change_p:text(radio) {label "Text Footer changeable?"} {options {{"Yes" t} {"No" f}}}}
	    {upload_file:file,optional {label "CSS-File"}}
	}
    }
    "text/html" {
	# html content mail-class
	ad_form -extend -name mail_class_form -form {
	    {html_header:text(textarea),optional {label "Default HTML Header"} {html {rows 5 cols 80}}}
	    {html_header_change_p:text(radio) {label "HTML Header changeable?"} {options {{"Yes" t} {"No" f}}}}
	    {html_body:text(textarea),optional {label "Default HTML Body"} {html {rows 5 cols 80}}}
	    {html_body_change_p:text(radio) {label "HTML Body changeable?"} {options {{"Yes" t} {"No" f}}}}
	    {html_footer:text(textarea),optional {label "Default HTML Footer"} {html {rows 5 cols 80}}}
	    {html_footer_change_p:text(radio) {label "HTML Footer changeable?"} {options {{"Yes" t} {"No" f}}}}
	    {upload_file:file,optional {label "CSS-File"}}
	}
    }
}

if {[exists_and_not_null css_file]} {
    ad_form -extend -name mail_class_form -form {
	{delete_css:integer(checkbox),optional {label "Delete CSS File"} {options $css_file}}
    }
}

ad_form -extend -name mail_class_form -form {
    {comments:text(textarea),optional {label "Comments"} {html {rows 5 cols 80}}}
} -new_request {
    if {[exists_and_not_null copy_class_id]} {
	permission::require_permission -object_id $copy_class_id -privilege read

	db_1row get_copy_class_data {}
    } else {
	set name ""
	set locale [ad_conn locale]
	set sender_email [ad_system_owner]
	set subject ""
	set subject_change_p t
	set text_header ""
	set text_header_change_p t
	set text_body ""
	set text_body_change_p t
	set text_footer ""
	set text_footer_change_p t
	set html_header ""
	set html_header_change_p t
	set html_body ""
	set html_body_change_p t
	set html_footer ""
	set html_footer_change_p t
	set comments ""
    }
} -edit_request {
    permission::require_permission -object_id $class_id -privilege admin

    db_1row get_class_data {}
} -on_submit {
    if {$mime_type == "text/html"} {
	set text_header ""
	set text_header_change_p t
	set text_body ""
	set text_body_change_p t
	set text_footer ""
	set text_footer_change_p t
    }
    if {$mime_type == "text/plain"} {
	set html_header ""
	set html_header_change_p t
	set html_body ""
	set html_body_change_p t
	set html_footer ""
	set html_footer_change_p t
    }
} -new_data {
    set class_id [mail_class::add -name $name -locale $locale \
		      -sender_email $sender_email -subject $subject \
		      -subject_change_p $subject_change_p -text_header $text_header \
		      -text_header_change_p $text_header_change_p -text_body $text_body \
		      -text_body_change_p $text_body_change_p -text_footer $text_footer \
		      -text_footer_change_p $text_footer_change_p -html_header $html_header \
		      -html_header_change_p $html_header_change_p -html_body $html_body \
		      -html_body_change_p $html_body_change_p -html_footer $html_footer \
		      -html_footer_change_p $html_footer_change_p -mime_type $mime_type \
		      -comments $comments]
} -edit_data {
    permission::require_permission -object_id $class_id -privilege admin

    mail_class::edit -mail_class_id $class_id -name $name -locale $locale \
	-sender_email $sender_email -subject $subject -subject_change_p $subject_change_p \
	-text_header $text_header -text_header_change_p $text_header_change_p \
	-text_body $text_body -text_body_change_p $text_body_change_p \
	-text_footer $text_footer -text_footer_change_p $text_footer_change_p \
	-html_header $html_header -html_header_change_p $html_header_change_p \
	-html_body $html_body -html_body_change_p $html_body_change_p \
	-html_footer $html_footer -html_footer_change_p $html_footer_change_p \
	-mime_type $mime_type -comments $comments
} -after_submit {
    db_transaction {
	if {[info exists delete_css]} {
	    db_dml delete_css_file {}
	}
	if {![empty_string_p $upload_file]} {
	    set filename [lindex $upload_file 0]
	    set tmp_filename [lindex $upload_file 1]
	    set file_mimetype [lindex $upload_file 2]
	    set n_bytes [file size $tmp_filename]
	    set max_file_size [ad_parameter MaxAttachmentSize]

	    if { $n_bytes > $max_file_size && $max_file_size > 0 } {
		ad_return_complaint 1 "Your file is too large.  The publisher of
                 [ad_system_name] has chosen to limit attachments to
                 [util_commify_number $max_file_size] bytes.\n"
	    }
	    if { $n_bytes == 0 } {
		ad_return_complaint 1 "Your file is zero-length.  Either you attempted to
                 upload a zero length file, a file which does not exists, 
                 or something went wrong during the transfer.\n"
	    }

	    if {[db_0or1row check_css_file_exists {}]} {
		# there's already a css-file, so generate a new revision
		cr_import_content -item_id $item_id -title $filename $class_id $tmp_filename $n_bytes $file_mimetype $filename
	    } else {
		# generate new css-file
		cr_import_content -title $filename $class_id $tmp_filename $n_bytes $file_mimetype $filename
	    }
	    ns_unlink $tmp_filename
	    db_dml make_latest_revision_live_revision {}
	}
	if {[info exists copy_class_id] && [empty_string_p $upload_file] && ![exists_and_not_null delete_css]} {
	    # copy css-file if no file uploaded
	    if {[db_0or1row get_file_data_for_copy {}]} {
		# css-file exists in class to be copied from
		set old_file "[cr_fs_path][cr_create_content_file_path $item_id $revision_id]"
		set revision_id [cr_import_content -title $filename $class_id $old_file $content_length $file_mimetype $filename]
		cr_set_imported_content_live $file_mimetype $revision_id
	    }
	}
    }
    ad_returnredirect "."
    ad_script_abort
}

ad_return_template
